@@ -5,15 +5,17 @@ import numpy as np
 import pandas as pd
 from tabulate import tabulate
 import tensorflow as tf
+from tensorflow.python.ops.gen_batch_ops import batch
 
 from sknlp.vocab import Vocab
 from sknlp.data import TaggingDataset
 from sknlp.callbacks import ModelScoreCallback
+from sknlp.layers import CrfDecodeLayer
 
 from ..supervised_model import SupervisedNLPModel
 from ..text2vec import Text2vec
 
-from .utils import tagging_fscore
+from .utils import tagging_fscore, viterbi_decode
 
 
 class DeepTagger(SupervisedNLPModel):
@@ -28,7 +30,7 @@ class DeepTagger(SupervisedNLPModel):
         embedding_size: int = 100,
         text2vec: Optional[Text2vec] = None,
         loss: str = None,
-        **kwargs
+        **kwargs,
     ):
         classes = list(classes)
         if start_tag is not None and end_tag is not None:
@@ -49,8 +51,9 @@ class DeepTagger(SupervisedNLPModel):
             embedding_size=embedding_size,
             text2vec=text2vec,
             task="tagging",
-            **kwargs
+            **kwargs,
         )
+        self._decode_model = None
         self._loss = loss
 
     @property
@@ -69,13 +72,12 @@ class DeepTagger(SupervisedNLPModel):
         return None
 
     def get_callbacks(self, batch_size: int) -> List[tf.keras.callbacks.Callback]:
-        return [
-            ModelScoreCallback(
-                partial(self.score, batch_size=batch_size),
-                self.valid_X,
-                self.valid_y
-            )
-        ]
+        callbacks = []
+        if self.valid_dataset is not None:
+            callbacks.append(ModelScoreCallback(partial(
+                self.score, dataset=self.valid_dataset, batch_size=batch_size
+            )))
+        return callbacks
 
     def get_metrics(self) -> List[tf.keras.metrics.Metric]:
         return []
@@ -104,9 +106,14 @@ class DeepTagger(SupervisedNLPModel):
         X: Sequence[str] = None,
         *,
         dataset: TaggingDataset = None,
-        batch_size: int = 128
+        batch_size: int = 128,
     ) -> List[List[str]]:
-        tag_ids_list = super().predict(X, dataset=dataset, batch_size=batch_size)
+        emissions, mask = super().predict(X, dataset=dataset, batch_size=batch_size)
+        tag_ids_list = viterbi_decode(
+            self._model.get_layer("crf").get_weights()[0],
+            emissions.transpose(1, 0, 2),
+            mask.transpose(1, 0),
+        )
         exclude_tags = {self.pad_tag, self.start_tag, self.end_tag}
         predictions = []
         for tag_ids in tag_ids_list:
@@ -124,11 +131,14 @@ class DeepTagger(SupervisedNLPModel):
         self,
         X: Sequence[str] = None,
         y: Sequence[Sequence[str]] = None,
-        batch_size: int = 128
+        *,
+        dataset: TaggingDataset = None,
+        batch_size: int = 128,
     ) -> pd.DataFrame:
-        predictions = self.predict(X=X, batch_size=batch_size)
+        dataset = self.prepare_dataset(X, y, dataset)
+        predictions = self.predict(dataset=dataset, batch_size=batch_size)
         labels = list(set([c.split("-")[-1] for c in self.classes if "-" in c]))
-        return tagging_fscore(X, y, predictions, labels)
+        return tagging_fscore(dataset.text, dataset.label, predictions, labels)
 
     def format_score(self, score_df: pd.DataFrame, format: str = "markdown") -> str:
         return tabulate(score_df, headers="keys", tablefmt="github", showindex=False)
@@ -145,4 +155,10 @@ class DeepTagger(SupervisedNLPModel):
         config = super()._filter_config(config)
         config.pop("algorithm", None)
         config.pop("task", None)
-        return config
\ No newline at end of file
+        return config
+
+    def get_custom_objects(self) -> Dict[str, Any]:
+        return {
+            **super().get_custom_objects(),
+            "CrfDecodeLayer": CrfDecodeLayer,
+        }
