@@ -91,12 +91,9 @@ class Bert2vec(Text2vec):
         type_ids = tf.keras.Input(
             shape=(sequence_length,), dtype=tf.int64, name="input_type_ids"
         )
-        masks = tf.keras.Input(
-            shape=(sequence_length,), dtype=tf.int64, name="input_masks"
-        )
-        outputs = bert_layer([token_ids, type_ids, masks])
+        outputs = bert_layer([token_ids, type_ids])
         self._model = tf.keras.Model(
-            inputs=[token_ids, type_ids, masks], outputs=outputs, name=name
+            inputs=[token_ids, type_ids], outputs=outputs, name=name
         )
 
     def __call__(self, inputs: tf.Tensor) -> tf.Tensor:
